#ifndef _RAM_H
#define _RAM_H

#include "stm32f4xx.h"
#include "eeprom.h"

#define      RAM_SUCCESS       0
#define      RAM_BAD_ADDRESS  -1
#define      RAM_WRITE_ERROR  -2
#define      RAM_BAD_BIT      -3
#define      RAM_BAD_ACCESS   -4

typedef enum {
  BIOLOID_MODEL_NUMBER_L          = DEVICE_MODEL_OFFSET,
  BIOLOID_MODEL_NUMBER_H          = DEVICE_MODEL_OFFSET+1,
  BIOLOID_VERSION                 = FIRMWARE_VERSION_OFFSET,
  BIOLOID_ID                      = DEVICE_ID_OFFSET,
  BIOLOID_BAUD_RATE               = BAUDRATE_OFFSET,
  BIOLOID_RETURN_DELAY_TIME       = RETURN_DELAY_OFFSET,
  BIOLOID_MM_PERIOD               = MM_PERIOD_OFFSET,
  BIOLOID_RETURN_LEVEL            = RETURN_LEVEL_OFFSET,
  BIOLOID_DXL_POWER               = 0x18,
  BIOLOID_LED                     = 0x19,
  BIOLOID_PUSHBUTTON              = 0x1A,
  BIOLOID_MM_NUM_SERVOS           = 0x1B,
  BIOLOID_MM_CONTROL              = 0x1C,
  BIOLOID_MM_STATUS               = 0x1D,
  BIOLOID_MM_MODULE_EN0           = 0x1E,
  BIOLOID_MM_MODULE_EN1           = 0x1F,
  BIOLOID_MM_MODULE_EN2           = 0x20,
  BIOLOID_MM_MODULE_EN3           = 0x21,
  BIOLOID_MM_MODULE_EN4           = 0x22,
  BIOLOID_MM_MODULE_EN5           = 0x23,
  BIOLOID_MM_MODULE_EN6           = 0x24,
  BIOLOID_MM_MODULE_EN7           = 0x25,
  BIOLOID_MM_MODULE_EN8           = 0x26,
  BIOLOID_MM_MODULE_EN9           = 0x27,
  BIOLOID_MM_MODULE_EN10          = 0x28,
  BIOLOID_MM_MODULE_EN11          = 0x29,
  BIOLOID_MM_MODULE_EN12          = 0x2A,
  BIOLOID_MM_MODULE_EN13          = 0x2B,
  BIOLOID_MM_MODULE_EN14          = 0x2C,
  BIOLOID_MM_MODULE_EN15          = 0x2D,
  BIOLOID_MM_SERVO0_CUR_POS_L     = 0x2E,
  BIOLOID_MM_SERVO0_CUR_POS_H     = 0x2F,
  BIOLOID_MM_SERVO1_CUR_POS_L     = 0x30,
  BIOLOID_MM_SERVO1_CUR_POS_H     = 0x31,
  BIOLOID_MM_SERVO2_CUR_POS_L     = 0x32,
  BIOLOID_MM_SERVO2_CUR_POS_H     = 0x33,
  BIOLOID_MM_SERVO3_CUR_POS_L     = 0x34,
  BIOLOID_MM_SERVO3_CUR_POS_H     = 0x35,
  BIOLOID_MM_SERVO4_CUR_POS_L     = 0x36,
  BIOLOID_MM_SERVO4_CUR_POS_H     = 0x37,
  BIOLOID_MM_SERVO5_CUR_POS_L     = 0x38,
  BIOLOID_MM_SERVO5_CUR_POS_H     = 0x39,
  BIOLOID_MM_SERVO6_CUR_POS_L     = 0x3A,
  BIOLOID_MM_SERVO6_CUR_POS_H     = 0x3B,
  BIOLOID_MM_SERVO7_CUR_POS_L     = 0x3C,
  BIOLOID_MM_SERVO7_CUR_POS_H     = 0x3D,
  BIOLOID_MM_SERVO8_CUR_POS_L     = 0x3E,
  BIOLOID_MM_SERVO8_CUR_POS_H     = 0x3F,
  BIOLOID_MM_SERVO9_CUR_POS_L     = 0x40,
  BIOLOID_MM_SERVO9_CUR_POS_H     = 0x41,
  BIOLOID_MM_SERVO10_CUR_POS_L    = 0x42,
  BIOLOID_MM_SERVO10_CUR_POS_H    = 0x43,
  BIOLOID_MM_SERVO11_CUR_POS_L    = 0x44,
  BIOLOID_MM_SERVO11_CUR_POS_H    = 0x45,
  BIOLOID_MM_SERVO12_CUR_POS_L    = 0x46,
  BIOLOID_MM_SERVO12_CUR_POS_H    = 0x47,
  BIOLOID_MM_SERVO13_CUR_POS_L    = 0x48,
  BIOLOID_MM_SERVO13_CUR_POS_H    = 0x49,
  BIOLOID_MM_SERVO14_CUR_POS_L    = 0x4A,
  BIOLOID_MM_SERVO14_CUR_POS_H    = 0x4B,
  BIOLOID_MM_SERVO15_CUR_POS_L    = 0x4C,
  BIOLOID_MM_SERVO15_CUR_POS_H    = 0x4D,
  BIOLOID_MM_SERVO16_CUR_POS_L    = 0x4E,
  BIOLOID_MM_SERVO16_CUR_POS_H    = 0x4F,
  BIOLOID_MM_SERVO17_CUR_POS_L    = 0x50,
  BIOLOID_MM_SERVO17_CUR_POS_H    = 0x51,
  BIOLOID_MM_SERVO18_CUR_POS_L    = 0x52,
  BIOLOID_MM_SERVO18_CUR_POS_H    = 0x53,
  BIOLOID_MM_SERVO19_CUR_POS_L    = 0x54,
  BIOLOID_MM_SERVO19_CUR_POS_H    = 0x55,
  BIOLOID_MM_SERVO20_CUR_POS_L    = 0x56,
  BIOLOID_MM_SERVO20_CUR_POS_H    = 0x57,
  BIOLOID_MM_SERVO21_CUR_POS_L    = 0x58,
  BIOLOID_MM_SERVO21_CUR_POS_H    = 0x59,
  BIOLOID_MM_SERVO22_CUR_POS_L    = 0x5A,
  BIOLOID_MM_SERVO22_CUR_POS_H    = 0x5B,
  BIOLOID_MM_SERVO23_CUR_POS_L    = 0x5C,
  BIOLOID_MM_SERVO23_CUR_POS_H    = 0x5D,
  BIOLOID_MM_SERVO24_CUR_POS_L    = 0x5E,
  BIOLOID_MM_SERVO24_CUR_POS_H    = 0x5F,
  BIOLOID_MM_SERVO25_CUR_POS_L    = 0x60,
  BIOLOID_MM_SERVO25_CUR_POS_H    = 0x61,
  BIOLOID_MM_SERVO26_CUR_POS_L    = 0x62,
  BIOLOID_MM_SERVO26_CUR_POS_H    = 0x63,
  BIOLOID_MM_SERVO27_CUR_POS_L    = 0x64,
  BIOLOID_MM_SERVO27_CUR_POS_H    = 0x65,
  BIOLOID_MM_SERVO28_CUR_POS_L    = 0x66,
  BIOLOID_MM_SERVO28_CUR_POS_H    = 0x67,
  BIOLOID_MM_SERVO29_CUR_POS_L    = 0x68,
  BIOLOID_MM_SERVO29_CUR_POS_H    = 0x69,
  BIOLOID_MM_SERVO30_CUR_POS_L    = 0x6A,
  BIOLOID_MM_SERVO30_CUR_POS_H    = 0x6B,
  BIOLOID_IMU_STATUS              = 0x6C,
  BIOLOID_IMU_CONTROL             = 0x6D,
  BIOLOID_IMU_ACCEL_X_L           = 0x6E,
  BIOLOID_IMU_ACCEL_X_H           = 0x6F,
  BIOLOID_IMU_ACCEL_Y_L           = 0x70,
  BIOLOID_IMU_ACCEL_Y_H           = 0x71,
  BIOLOID_IMU_ACCEL_Z_L           = 0x72,
  BIOLOID_IMU_ACCEL_Z_H           = 0x73,
  BIOLOID_IMU_GYRO_X_L            = 0x74,
  BIOLOID_IMU_GYRO_X_H            = 0x75,
  BIOLOID_IMU_GYRO_Y_L            = 0x76,
  BIOLOID_IMU_GYRO_Y_H            = 0x77,
  BIOLOID_IMU_GYRO_Z_L            = 0x78,
  BIOLOID_IMU_GYRO_Z_H            = 0x79,
  BIOLOID_IMU_COMPASS_X_L         = 0x7A,
  BIOLOID_IMU_COMPASS_X_H         = 0x7B,
  BIOLOID_IMU_COMPASS_Y_L         = 0x7C,
  BIOLOID_IMU_COMPASS_Y_H         = 0x7D,
  BIOLOID_IMU_COMPASS_Z_L         = 0x7E,
  BIOLOID_IMU_COMPASS_Z_H         = 0x7F,
  BIOLOID_IMU_TEMP                = 0x80,
  BIOLOID_ACTION_PAGE             = 0x81,
  BIOLOID_ACTION_CONTROL          = 0x82,
  BIOLOID_ACTION_SATUTUS          = 0x83
} bioloid_registers;

#define      RAM_SIZE          256

extern uint8_t ram_data[RAM_SIZE];

void ram_init(void);
inline void ram_read_byte(uint8_t address, uint8_t *data);
inline void ram_read_word(uint8_t address, uint16_t *data);
uint8_t ram_read_table(uint8_t address, uint8_t length,uint8_t *data);
uint8_t ram_set_bit(uint8_t address, uint8_t bit);
uint8_t ram_clear_bit(uint8_t address, uint8_t bit);
uint8_t ram_write_byte(uint8_t address, uint8_t data);
uint8_t ram_write_word(uint8_t address, uint16_t data);
uint8_t ram_write_table(uint8_t address, uint8_t length,uint8_t *data);
inline uint8_t ram_in_range(uint8_t start_reg,uint8_t end_reg,uint8_t address,uint8_t length);

#endif
