#include "action.h"
#include "ram.h"

#define SPEED_BASE_SCHEDULE 0x00
#define TIME_BASE_SCHEDULE  0x0A

// private variables
typedef enum {ACTION_PRE,ACTION_MAIN,ACTION_POST,ACTION_PAUSE} action_states;

TPage action_next_page;
TPage action_current_page;
// angle variables
int64_t action_moving_angles[PAGE_MAX_NUM_SERVOS];// fixed point 48|16 format
int64_t action_current_angles[PAGE_MAX_NUM_SERVOS];
int64_t action_start_angles[PAGE_MAX_NUM_SERVOS];// fixed point 48|16 format
// speed variables
int64_t action_start_speed[PAGE_MAX_NUM_SERVOS];// fixed point 48|16 format
int64_t action_main_speed[PAGE_MAX_NUM_SERVOS];// fixed point 48|16 format
// control variables
uint8_t action_end,action_stop;
uint8_t action_zero_speed_finish[PAGE_MAX_NUM_SERVOS];
uint8_t action_next_index;
uint8_t action_running;
// time variables (in time units (7.8 ms each time unit))
int64_t action_total_time;// fixed point 48|16 format
int64_t action_pre_time;// fixed point 48|16 format
int64_t action_main_time;// fixed point 48|16 format
int64_t action_step_time;// fixed point 48|16 format
int64_t action_pause_time;// fixed point 48|16 format
int64_t action_current_time;// fixed point 48|16 format
int64_t action_section_time;// fixed point 48|16 format
int64_t action_period;

// private functions
void action_load_next_step(void)
{
  // page and step information
  static uint8_t current_index=0;
  static uint8_t current_step_index=0;
  static int8_t num_repetitions=0;
  // angle variables
  int16_t next_angle;// information from the flash memory (fixed point 9|7 format)
  int64_t max_angle;// fixed point format 48|16 format
  int64_t tmp_angle;// fixed point format 48|16 format
  int64_t angle;// fixed point format 48|16 format
  int64_t target_angles;// fixed point 48|16 format
  int64_t next_angles;// fixed point 48|16 format
  int64_t action_pre_time_initial_speed_angle;// fixed point 48|16 format
  int64_t moving_angle;// fixed point 48|16 format
  // time variables
  int64_t accel_time_num;// fixed point 48|16 format
  int64_t zero_speed_divider;// fixed point 48|16 format
  int64_t non_zero_speed_divider;// fixed point 48|16 format
  // control variables
  uint8_t dir_change;
  // control information
  uint8_t i=0;

  current_step_index++;
  if(current_step_index>=action_current_page.header.stepnum)// the last step has been executed
  {
    pages_copy_page(&action_next_page,&action_current_page);// make the next page active
    if(current_index!=action_next_index)
      num_repetitions=action_current_page.header.repeat;
    current_step_index=0;
    current_index=action_next_index;
  }
  else if(current_step_index==action_current_page.header.stepnum-1)// it is the last step
  {
    if(action_stop)
      action_next_index=action_current_page.header.exit;
    else
    {
      num_repetitions--;
      if(num_repetitions>0)
        action_next_index=current_index;
      else
        action_next_index=action_current_page.header.next;
    }
    if(action_next_index==0)
      action_end=0x01;
    else
    {
      if(action_next_index!=current_index)
      {
        pages_get_page(action_next_index,&action_next_page);
        if(!pages_check_checksum(&action_next_page))
          action_end=0x01;
      }
      if(action_next_page.header.repeat==0 || action_next_page.header.stepnum==0)
        action_end=0x01;
    }
  }
  // compute trajectory
  action_pause_time=((action_current_page.steps[current_step_index].pause<<14)/action_current_page.header.speed);
  action_step_time=((action_current_page.steps[current_step_index].time<<14)/action_current_page.header.speed);
  if(action_step_time<action_period)
    action_step_time=action_period;// 0.078 in 16|16 format
  max_angle=0;
  for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
  {
    angle=action_current_page.steps[current_step_index].position[i];
    if(angle==0x5A00)// bigger than 180
      target_angles=action_current_angles[i];
    else
      target_angles=angle<<9;
    action_moving_angles[i]=target_angles-action_current_angles[i];
    if(action_end)
      next_angles=target_angles;
    else
    {
      if(current_step_index==action_current_page.header.stepnum-1)
        next_angle=action_next_page.steps[0].position[i];
      else
        next_angle=action_current_page.steps[current_step_index+1].position[i];
      if(next_angle==0x5A00)
        next_angles=target_angles;
      else
        next_angles=next_angle<<9;
    }
    // check changes in direction
    if(((action_current_angles[i] < target_angles) && (target_angles < next_angles)) ||
       ((action_current_angles[i] > target_angles) && (target_angles > next_angles)))
      dir_change=0x00;
    else
      dir_change=0x01;
    // check the type of ending
    if(dir_change || action_pause_time>0 || action_end)
      action_zero_speed_finish[i]=0x01;
    else
      action_zero_speed_finish[i]=0x00;
    // find the maximum angle of motion in speed base schedule
    if(action_current_page.header.schedule==SPEED_BASE_SCHEDULE)// speed base schedule
    {
      // fer el valor absolut
      if(action_moving_angles[i]<0)
        tmp_angle=-action_moving_angles[i];
      else
        tmp_angle=action_moving_angles[i];
      if(tmp_angle>max_angle)
        max_angle=tmp_angle;
    }
  }
  if(action_current_page.header.schedule==SPEED_BASE_SCHEDULE)
    action_step_time=(max_angle*256)/(action_current_page.steps[current_step_index].time*720);
  accel_time_num=action_current_page.header.accel;
  accel_time_num=accel_time_num<<9;
  if(action_step_time<=(accel_time_num<<1))
  {
    if(action_step_time==0)
      accel_time_num=0;
    else
    {
      accel_time_num=(action_step_time-action_period)>>1;
      if(accel_time_num==0)
        action_step_time=0;
    }
  }
  action_total_time=action_step_time;
  action_pre_time=accel_time_num;
  action_main_time=action_total_time-action_pre_time;
  non_zero_speed_divider=(action_pre_time>>1)+action_main_time;
  if(non_zero_speed_divider<action_period)
    non_zero_speed_divider=action_period;
  zero_speed_divider=action_main_time;
  if(zero_speed_divider<action_period)
    zero_speed_divider=action_period;
  for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
  {
    action_pre_time_initial_speed_angle=(action_start_speed[i]*action_pre_time)>>1;
    moving_angle=action_moving_angles[i]<<16;
    if(action_zero_speed_finish[i])
      action_main_speed[i]=(moving_angle-action_pre_time_initial_speed_angle)/zero_speed_divider;
    else
      action_main_speed[i]=(moving_angle-action_pre_time_initial_speed_angle)/non_zero_speed_divider;
//    if((action_main_speed[i]>>16)>info.max_speed)
//      action_main_speed[i]=(info.max_speed*65536);
//    else if((action_main_speed[i]>>16)<-info.max_speed)
//      action_main_speed[i]=-(info.max_speed*65536);
  }
}

// public functions
void action_init(uint16_t period)
{
  unsigned char i;

  // init current_angles with the current position of the servos
  for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
  {
    // angle variables
    action_moving_angles[i]=0;// fixed point 48|16 format
    action_start_angles[i]=action_current_angles[i];
    // speed variables
    action_start_speed[i]=0;// fixed point 48|16 format
    action_main_speed[i]=0;// fixed point 48|16 format
    // control variables
    action_zero_speed_finish[i]=0x00;
  }
  // clear the contents of the next page
  pages_clear_page(&action_next_page);
  pages_clear_page(&action_current_page);
  // control variables
  action_end=0x00;
  action_stop=0x00;
  action_running=0x00;
  action_next_index=0;
  // time variables (in time units (7.8 ms each time unit))
  action_total_time=0;// fixed point 48|16 format
  action_pre_time=0;// fixed point 48|16 format
  action_main_time=0;// fixed point 48|16 format
  action_step_time=0;// fixed point 48|16 format
  action_pause_time=0;// fixed point 48|16 format
  action_current_time=0;// fixed point 48|16 format
  action_section_time=0;// fixed point 48|16 format

  action_period=period;
}

inline void action_set_period(uint16_t period)
{
  action_period=period;
}

inline uint16_t action_get_period(void)
{
  return action_period;
}

uint8_t action_load_page(uint8_t page_id)
{
  action_next_index=page_id;
  if(action_next_index<0 || action_next_index>255)
    return 0x00;
  pages_get_page(action_next_index,&action_next_page);
  pages_clear_page(&action_current_page);
  if(!pages_check_checksum(&action_next_page))
    return 0x00;

  return 0x01;
}

void action_start_page(void)
{
  uint8_t i;

  for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
    action_start_angles[i]=action_current_angles[i];
  action_stop=0x00;
  action_current_time=0;
  action_section_time=0; 
  ram_set_bit(BIOLOID_ACTION_SATUTUS,0);
  action_running=0x01;
}

void action_stop_page(void)
{
  action_stop=0x01;
}

uint8_t action_is_running(void)
{
  return action_running;
}

// motion manager interface
void action_process(void)
{
  uint8_t i;
  // angle variables 
  static int64_t accel_angles[PAGE_MAX_NUM_SERVOS];// fixed point 48|16 format
  static int64_t main_angles[PAGE_MAX_NUM_SERVOS];// fixed point 48|16 format
  // speed variables (in controller units)
  static int64_t current_speed[PAGE_MAX_NUM_SERVOS]={0};// fixed point 48|16 format
  int64_t delta_speed;
  // control variables 
  static action_states state=ACTION_PAUSE;

  if(action_running==0x01)
  {
    action_current_time+=action_period;
    switch(state)
    {
      case ACTION_PRE: if(action_current_time>action_section_time)
                       {
                         for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
                         {
                           /* the last segment of the pre section */
                           delta_speed=(action_main_speed[i]-action_start_speed[i]);
                           current_speed[i]=action_start_speed[i]+delta_speed;
                           accel_angles[i]=(action_start_speed[i]*action_section_time+((delta_speed*action_section_time)>>1))>>16;
                           action_current_angles[i]=action_start_angles[i]+accel_angles[i];
                           /* update of the state */
                           if(!action_zero_speed_finish[i])
                           {
                             if((action_step_time-action_pre_time)==0)
                               main_angles[i]=0;
                             else
                               main_angles[i]=((action_moving_angles[i]-accel_angles[i])*(action_step_time-(action_pre_time<<1)))/(action_step_time-action_pre_time);
                             action_start_angles[i]=action_current_angles[i];
                           }
                           else
                           {
                             main_angles[i]=action_moving_angles[i]-accel_angles[i]-(((action_main_speed[i]*action_pre_time)>>1)>>16);
                             action_start_angles[i]=action_current_angles[i];
                           }
                           /* the first step of the main section */
                           action_current_angles[i]=action_start_angles[i]+(main_angles[i]*(action_current_time-action_section_time))/(action_step_time-(action_pre_time<<1));
                           current_speed[i]=action_main_speed[i];
                         }
                         action_current_time=action_current_time-action_section_time;
                         action_section_time=action_step_time-(action_pre_time<<1);
                         state=ACTION_MAIN;
                       }
                       else
                       {
                         for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
                         {
                           delta_speed=((action_main_speed[i]-action_start_speed[i])*action_current_time)/action_section_time;
                           current_speed[i]=action_start_speed[i]+delta_speed;
                           accel_angles[i]=((action_start_speed[i]*action_current_time)+((delta_speed*action_current_time)>>1))>>16;
                           action_current_angles[i]=action_start_angles[i]+accel_angles[i];
                         }
                         state=ACTION_PRE;
                       }
                       break;
      case ACTION_MAIN: if(action_current_time>action_section_time)
                        {
                          for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
                          {
                            /* last segment of the main section */
                            action_current_angles[i]=action_start_angles[i]+main_angles[i];
                            current_speed[i]=action_main_speed[i];
                            /* update state */
                            action_start_angles[i]=action_current_angles[i];
                            main_angles[i]=action_moving_angles[i]-main_angles[i]-accel_angles[i];
                            /* first segment of the post section */
                            if(action_zero_speed_finish[i])
                            {
                              delta_speed=((0.0-action_main_speed[i])*(action_current_time-action_section_time))/action_pre_time;
                              current_speed[i]=action_main_speed[i]+delta_speed;
                              action_current_angles[i]=action_start_angles[i]+(((action_main_speed[i]*(action_current_time-action_section_time))+((delta_speed*(action_current_time-action_section_time))>>1))>>16);
                            }
                            else
                            {
                              action_current_angles[i]=action_start_angles[i]+((main_angles[i]*(action_current_time-action_section_time))/action_pre_time);
                              current_speed[i]=action_main_speed[i];
                            }
                          }
                          action_current_time=action_current_time-action_section_time;
                          action_section_time=action_pre_time;
                          state=ACTION_POST;
                        }
                        else
                        {
                          for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
                          {
                            action_current_angles[i]=action_start_angles[i]+(main_angles[i]*action_current_time)/action_section_time;
                            current_speed[i]=action_main_speed[i];
                          }
                          state=ACTION_MAIN;
                        }
                        break;
      case ACTION_POST: if(action_current_time>action_section_time)
                        {
                          for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
                          {
                            /* last segment of the post section */
                            if(action_zero_speed_finish[i])
                            {
                              delta_speed=-action_main_speed[i];
                              current_speed[i]=action_main_speed[i]+delta_speed;
                              action_current_angles[i]=action_start_angles[i]+(((action_main_speed[i]*action_section_time)+((delta_speed*action_section_time)>>1))>>16);
                            }
                            else
                            {
                              action_current_angles[i]=action_start_angles[i]+main_angles[i];
                              current_speed[i]=action_main_speed[i];
                            }
                            /* update state */
                            action_start_angles[i]=action_current_angles[i];
                            action_start_speed[i]=current_speed[i];
                          }
                          /* load the next step */
                          if(action_pause_time==0)
                          {
                            if(action_end)
                            {
                              action_load_next_step();
                              state=ACTION_PAUSE;
                              action_end=0x00;
                              ram_clear_bit(BIOLOID_ACTION_SATUTUS,0);
                              action_running=0x00;
                            }
                            else
                            {
                              action_load_next_step();
                              /* first step of the next section */
                              for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
                              {
                                delta_speed=((action_main_speed[i]-action_start_speed[i])*(action_current_time-action_section_time))/action_pre_time;
                                current_speed[i]=action_start_speed[i]+delta_speed;
                                accel_angles[i]=(((action_start_speed[i]*(action_current_time-action_section_time))+((delta_speed*(action_current_time-action_section_time))>>1))>>16);
                                action_current_angles[i]=action_start_angles[i]+accel_angles[i];
                              }
                              action_current_time=action_current_time-action_section_time;
                              action_section_time=action_pre_time;
                              state=ACTION_PRE;
                            }
                          }
                          else
                          {
                            action_current_time=action_current_time-action_section_time;
                            action_section_time=action_pause_time;
                            state=ACTION_PAUSE;
                          }
                        }
                        else
                        {
                          for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
                          {
                            if(action_zero_speed_finish[i])
                            {
                              delta_speed=((0.0-action_main_speed[i])*action_current_time)/action_section_time;
                              current_speed[i]=action_main_speed[i]+delta_speed;
                              action_current_angles[i]=action_start_angles[i]+(((action_main_speed[i]*action_current_time)+((delta_speed*action_current_time)>>1))>>16);
                            }
                            else
                            {
                              action_current_angles[i]=action_start_angles[i]+(main_angles[i]*action_current_time)/action_section_time;
                              current_speed[i]=action_main_speed[i];
                            }
                          }
                          state=ACTION_POST;
                        }
                        break;
      case ACTION_PAUSE: if(action_current_time>action_section_time)
                         {
                           if(action_end)
                           {
                             // find next page to execute
                             action_load_next_step();
                             state=ACTION_PAUSE;
                             action_end=0x00;
                             ram_clear_bit(BIOLOID_ACTION_SATUTUS,0);
                             action_running=0x00;
                           }
                           else
                           {
                             // find next page to execute
                             action_load_next_step();
                             /* first step of the next section */
                             for(i=0;i<PAGE_MAX_NUM_SERVOS;i++)
                             {
                               delta_speed=((action_main_speed[i]-action_start_speed[i])*(action_current_time-action_section_time))/action_pre_time;
                               current_speed[i]=action_start_speed[i]+delta_speed;
                               accel_angles[i]=(((action_start_speed[i]*(action_current_time-action_section_time))+((delta_speed*(action_current_time-action_section_time))>>1))>>16);
                               action_current_angles[i]=action_start_angles[i]+accel_angles[i];
                             }
                             action_current_time=action_current_time-action_section_time;
                             action_section_time=action_pre_time;
                             state=ACTION_PRE;
                           }
                         }
                         else// pause section still active
                           state=ACTION_PAUSE;
                         break;
      default: break;
    }
  }
}

