#include "dynamixel_slave_uart_dma.h"

#define     DYN_SLAVE                    USART3
#define     DYN_SLAVE_CLK                RCC_APB1Periph_USART3
#define     DYN_SLAVE_CLK_INIT           RCC_APB1PeriphClockCmd
#define     DYN_SLAVE_IRQn               USART3_IRQn
#define     DYN_SLAVE_IRQHandler         USART3_IRQHandler

#define     DYN_SLAVE_TX_PIN             GPIO_Pin_10                
#define     DYN_SLAVE_TX_GPIO_PORT       GPIOC
#define     DYN_SLAVE_TX_GPIO_CLK        RCC_AHB1Periph_GPIOC
#define     DYN_SLAVE_TX_SOURCE          GPIO_PinSource10
#define     DYN_SLAVE_TX_AF              GPIO_AF_USART3

#define     DYN_SLAVE_RX_PIN             GPIO_Pin_11
#define     DYN_SLAVE_RX_GPIO_PORT       GPIOC
#define     DYN_SLAVE_RX_GPIO_CLK        RCC_AHB1Periph_GPIOB
#define     DYN_SLAVE_RX_SOURCE          GPIO_PinSource11
#define     DYN_SLAVE_RX_AF              GPIO_AF_USART3

/* DMA configuration */
#define     DYN_SLAVE_DR_ADDRESS         ((uint32_t)USART3 + 0x04) 
#define     DYN_SLAVE_DMA                DMA1
#define     DYN_SLAVE_DMA_CLK            RCC_AHB1Periph_DMA1

#define     DYN_SLAVE_TX_DMA_CHANNEL     DMA_Channel_4
#define     DYN_SLAVE_TX_DMA_STREAM      DMA1_Stream3
#define     DYN_SLAVE_TX_DMA_FLAG_FEIF   DMA_FLAG_FEIF3
#define     DYN_SLAVE_TX_DMA_FLAG_DMEIF  DMA_FLAG_DMEIF3
#define     DYN_SLAVE_TX_DMA_FLAG_TEIF   DMA_FLAG_TEIF3
#define     DYN_SLAVE_TX_DMA_FLAG_HTIF   DMA_FLAG_HTIF3
#define     DYN_SLAVE_TX_DMA_FLAG_TCIF   DMA_FLAG_TCIF3

#define     DYN_SLAVE_RX_DMA_CHANNEL     DMA_Channel_4
#define     DYN_SLAVE_RX_DMA_STREAM      DMA1_Stream1
#define     DYN_SLAVE_RX_DMA_FLAG_FEIF   DMA_FLAG_FEIF1
#define     DYN_SLAVE_RX_DMA_FLAG_DMEIF  DMA_FLAG_DMEIF1
#define     DYN_SLAVE_RX_DMA_FLAG_TEIF   DMA_FLAG_TEIF1
#define     DYN_SLAVE_RX_DMA_FLAG_HTIF   DMA_FLAG_HTIF1
#define     DYN_SLAVE_RX_DMA_FLAG_TCIF   DMA_FLAG_TCIF1

#define     DYN_SLAVE_DMA_TX_IRQn        DMA1_Stream3_IRQn
#define     DYN_SLAVE_DMA_RX_IRQn        DMA1_Stream1_IRQn
#define     DYN_SLAVE_DMA_TX_IRQHandler  DMA1_Stream3_IRQHandler
#define     DYN_SLAVE_DMA_RX_IRQHandler  DMA1_Stream1_IRQHandler

#define     MAX_BUFFER_LEN                1024
// private variables
uint8_t dyn_slave_address;// this module slave address
uint8_t dyn_slave_return_level;// type of response
uint8_t dyn_slave_return_delay;// delay in th response
// input buffer
uint8_t dyn_slave_rx_buffer[MAX_BUFFER_LEN];
volatile uint8_t dyn_slave_receiving_header;
// output buffer
uint8_t dyn_slave_tx_buffer[MAX_BUFFER_LEN];
// instruction packet ready flag
volatile uint8_t dyn_slave_packet_ready;
// sending status packet flag
volatile uint8_t dyn_slave_sending_packet;
// DMA initialization data structures
DMA_InitTypeDef  DYN_SLAVE_DMA_TX_InitStructure;
DMA_InitTypeDef  DYN_SLAVE_DMA_RX_InitStructure;

// private functions

// interrupt handlers
void DYN_SLAVE_IRQHandler(void)
{
  if(USART_GetITStatus(DYN_SLAVE, USART_IT_TC) != RESET)
  {
    USART_ClearFlag(DYN_SLAVE,USART_FLAG_TC);
    USART_ITConfig(DYN_SLAVE, USART_IT_TC, DISABLE);
    dyn_slave_sending_packet=0x00;
    // set up the DMA RX transaction
    DYN_SLAVE_DMA_RX_InitStructure.DMA_BufferSize = 4;
    DYN_SLAVE_DMA_RX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)dyn_slave_rx_buffer;
    DMA_Init(DYN_SLAVE_RX_DMA_STREAM,&DYN_SLAVE_DMA_RX_InitStructure);
    DMA_ITConfig(DYN_SLAVE_RX_DMA_STREAM,DMA_IT_TC,ENABLE);
    DMA_Cmd(DYN_SLAVE_RX_DMA_STREAM,ENABLE);
    USART_DMACmd(DYN_SLAVE, USART_DMAReq_Rx, ENABLE);
    dyn_slave_receiving_header=0x01;
  }
}

void DYN_SLAVE_DMA_TX_IRQHandler(void)
{
  DMA_ClearFlag(DYN_SLAVE_TX_DMA_STREAM,DYN_SLAVE_TX_DMA_FLAG_TCIF);
  DMA_ClearITPendingBit(DYN_SLAVE_TX_DMA_STREAM,DYN_SLAVE_TX_DMA_FLAG_TCIF);
  USART_ITConfig(DYN_SLAVE, USART_IT_TC, ENABLE);
}

void DYN_SLAVE_DMA_RX_IRQHandler(void)
{
  if(dyn_slave_receiving_header==0x01)
  {
    DMA_ClearFlag(DYN_SLAVE_RX_DMA_STREAM,DYN_SLAVE_RX_DMA_FLAG_TCIF);
    DMA_ClearITPendingBit(DYN_SLAVE_RX_DMA_STREAM,DYN_SLAVE_RX_DMA_FLAG_TCIF);
    DYN_SLAVE_DMA_RX_InitStructure.DMA_BufferSize = dyn_get_length(dyn_slave_rx_buffer);
    DYN_SLAVE_DMA_RX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&dyn_slave_rx_buffer[4];
    DMA_Init(DYN_SLAVE_RX_DMA_STREAM,&DYN_SLAVE_DMA_RX_InitStructure);
    DMA_Cmd(DYN_SLAVE_RX_DMA_STREAM,ENABLE);
    USART_DMACmd(DYN_SLAVE, USART_DMAReq_Rx, ENABLE);
    dyn_slave_receiving_header=0x00;
  }
  else
  {
    DMA_ClearFlag(DYN_SLAVE_RX_DMA_STREAM,DYN_SLAVE_RX_DMA_FLAG_TCIF);
    DMA_ClearITPendingBit(DYN_SLAVE_RX_DMA_STREAM,DYN_SLAVE_RX_DMA_FLAG_TCIF);
    DMA_ITConfig(DYN_SLAVE_RX_DMA_STREAM,DMA_IT_TC,DISABLE);
    dyn_slave_packet_ready=0x01;
  }
}

// public functions
void dyn_slave_init(void)
{
  uint16_t i;
  GPIO_InitTypeDef GPIO_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;
  USART_InitTypeDef USART_InitStructure;

  /* Enable GPIO clock */
  DYN_SLAVE_CLK_INIT(DYN_SLAVE_CLK, ENABLE);
  RCC_AHB1PeriphClockCmd(DYN_SLAVE_TX_GPIO_CLK | DYN_SLAVE_RX_GPIO_CLK, ENABLE);
  // configure the GPIO pins

  /* Connect USART pins to AF7 */
  GPIO_PinAFConfig(DYN_SLAVE_TX_GPIO_PORT, DYN_SLAVE_TX_SOURCE, DYN_SLAVE_TX_AF);
  GPIO_PinAFConfig(DYN_SLAVE_RX_GPIO_PORT, DYN_SLAVE_RX_SOURCE, DYN_SLAVE_RX_AF);

  /* Configure USART Tx and Rx as alternate function push-pull */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Pin = DYN_SLAVE_TX_PIN;
  GPIO_Init(DYN_SLAVE_TX_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = DYN_SLAVE_RX_PIN;
  GPIO_Init(DYN_SLAVE_RX_GPIO_PORT, &GPIO_InitStructure);

  // initialize the buffers
  for(i=0;i<MAX_BUFFER_LEN;i++)
  {
    dyn_slave_rx_buffer[i]=0x00;
    dyn_slave_tx_buffer[i]=0x00;
  }
  // initialize the flags
  dyn_slave_packet_ready=0x00;
  dyn_slave_sending_packet=0x00;
  dyn_slave_receiving_header=0x01;

  USART_DeInit(DYN_SLAVE);
  USART_StructInit(&USART_InitStructure);
  // configure the serial port
  USART_InitStructure.USART_BaudRate = 115200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  USART_Init(DYN_SLAVE, &USART_InitStructure);

  NVIC_InitStructure.NVIC_IRQChannel = DYN_SLAVE_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  USART_ITConfig(DYN_SLAVE, USART_IT_RXNE | USART_IT_ORE | USART_IT_TXE | USART_IT_CTS | USART_IT_LBD | USART_IT_IDLE | USART_IT_PE | USART_IT_ERR | USART_IT_TC, DISABLE);
  USART_ClearFlag(DYN_SLAVE,USART_FLAG_RXNE | USART_FLAG_ORE | USART_FLAG_TXE | USART_FLAG_CTS | USART_FLAG_LBD | USART_FLAG_IDLE | USART_FLAG_PE | USART_FLAG_TC);
  USART_ClearITPendingBit(DYN_SLAVE, USART_FLAG_RXNE | USART_FLAG_ORE | USART_FLAG_TXE | USART_FLAG_CTS | USART_FLAG_LBD | USART_FLAG_IDLE | USART_FLAG_PE | USART_FLAG_TC);
  /* Enable the DYN_SLAVE3 */
  USART_Cmd(DYN_SLAVE, ENABLE);

  // configure the DMA channels
  /* Configure TX DMA */
  RCC_AHB1PeriphClockCmd(DYN_SLAVE_DMA_CLK, ENABLE);
  DMA_DeInit(DYN_SLAVE_TX_DMA_STREAM);
  DYN_SLAVE_DMA_TX_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_1QuarterFull;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_Mode = DMA_Mode_Normal;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) (&(DYN_SLAVE->DR));
  DYN_SLAVE_DMA_TX_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_Priority = DMA_Priority_High;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_Channel = DYN_SLAVE_TX_DMA_CHANNEL;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_DIR = DMA_DIR_MemoryToPeripheral;
  DYN_SLAVE_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)dyn_slave_tx_buffer;
  DMA_Init(DYN_SLAVE_TX_DMA_STREAM,&DYN_SLAVE_DMA_TX_InitStructure);
  /* initialize DMA interrupts */
  NVIC_InitStructure.NVIC_IRQChannel = DYN_SLAVE_DMA_TX_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
  DMA_ITConfig(DYN_SLAVE_TX_DMA_STREAM,DMA_IT_TC,ENABLE);
  DMA_ITConfig(DYN_SLAVE_TX_DMA_STREAM,DMA_IT_HT | DMA_IT_TE | DMA_IT_FE | DMA_IT_DME,DISABLE);

  /* Configure RX DMA */
  DMA_DeInit(DYN_SLAVE_RX_DMA_STREAM);
  DYN_SLAVE_DMA_RX_InitStructure.DMA_BufferSize = 4;// transfer the first 3 bytes
  DYN_SLAVE_DMA_RX_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_Mode = DMA_Mode_Normal;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) (&(DYN_SLAVE->DR));
  DYN_SLAVE_DMA_RX_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_Priority = DMA_Priority_High;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_Channel = DYN_SLAVE_RX_DMA_CHANNEL;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)dyn_slave_rx_buffer;
  DMA_Init(DYN_SLAVE_RX_DMA_STREAM,&DYN_SLAVE_DMA_RX_InitStructure);
  /* initialize DMA interrupts */
  NVIC_InitStructure.NVIC_IRQChannel = DYN_SLAVE_DMA_RX_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
  DMA_ITConfig(DYN_SLAVE_RX_DMA_STREAM,DMA_IT_TC,ENABLE);
  DMA_ITConfig(DYN_SLAVE_RX_DMA_STREAM,DMA_IT_HT | DMA_IT_TE | DMA_IT_FE | DMA_IT_DME,DISABLE);

  DMA_Cmd(DYN_SLAVE_RX_DMA_STREAM,ENABLE);
  USART_DMACmd(DYN_SLAVE, USART_DMAReq_Rx, ENABLE);
}

inline void dyn_slave_set_address(uint8_t id)
{
  dyn_slave_address=id;  
}

inline uint8_t dyn_slave_get_address(void)
{
  return dyn_slave_address;
}

inline void dyn_slave_set_return_delay(uint8_t delay)
{
  dyn_slave_return_delay=delay;
}

inline void dyn_slave_set_return_level(uint8_t level)
{
  dyn_slave_return_level=level;
}

inline uint8_t dyn_slave_get_return_level(void)
{
  return dyn_slave_return_level;
}

uint8_t dyn_slave_is_packet_ready(void)
{
  return dyn_slave_packet_ready;
}

void dyn_slave_get_inst_packet(uint8_t *packet)
{
  uint8_t i;

  for(i=0;i<dyn_get_length(dyn_slave_rx_buffer)+4;i++)
    packet[i]=dyn_slave_rx_buffer[i];
  dyn_slave_packet_ready=0x00;
}

void dyn_slave_send_status_packet(uint8_t error,uint8_t length, uint8_t *data)
{
  // wait until the previous transmission has ended (if any)
  while(dyn_slave_sending_packet==0x01);
  // create the status packet
  dyn_init_status_packet(dyn_slave_tx_buffer);
  dyn_set_status_error(dyn_slave_tx_buffer,error);
  dyn_set_id(dyn_slave_tx_buffer,dyn_slave_address);
  dyn_set_status_data(dyn_slave_tx_buffer,length,data);
  dyn_set_checksum(dyn_slave_tx_buffer);
  // set the DMA transfer
  DMA_SetCurrDataCounter(DYN_SLAVE_TX_DMA_STREAM,dyn_get_length(dyn_slave_tx_buffer)+4);
  DMA_Cmd(DYN_SLAVE_TX_DMA_STREAM,ENABLE);
  USART_ClearFlag(DYN_SLAVE,USART_FLAG_TC);
  USART_DMACmd(DYN_SLAVE, USART_DMAReq_Tx, ENABLE);
  dyn_slave_sending_packet=0x01;
}

void dyn_slave_resend_status_packet(uint8_t *packet)
{
  // wait until the previous transmission has ended (if any)
  while(dyn_slave_sending_packet==0x01);
  // create the status packet
  dyn_init_status_packet(dyn_slave_tx_buffer);
  dyn_copy_packet(packet,dyn_slave_tx_buffer); 
  // set the DMA transfer
  DMA_SetCurrDataCounter(DYN_SLAVE_TX_DMA_STREAM,dyn_get_length(dyn_slave_tx_buffer)+4);
  DMA_Cmd(DYN_SLAVE_TX_DMA_STREAM,ENABLE);
  USART_ClearFlag(DYN_SLAVE,USART_FLAG_TC);
  USART_DMACmd(DYN_SLAVE, USART_DMAReq_Tx, ENABLE);
  dyn_slave_sending_packet=0x01;
}

void dyn_slave_reset(void)
{
  dyn_slave_sending_packet=0x00;
  // set up the DMA RX transaction
  DYN_SLAVE_DMA_RX_InitStructure.DMA_BufferSize = 4;
  DYN_SLAVE_DMA_RX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)dyn_slave_rx_buffer;
  DMA_Init(DYN_SLAVE_RX_DMA_STREAM,&DYN_SLAVE_DMA_RX_InitStructure);
  DMA_ITConfig(DYN_SLAVE_RX_DMA_STREAM,DMA_IT_TC,ENABLE);
  DMA_Cmd(DYN_SLAVE_RX_DMA_STREAM,ENABLE);
  USART_DMACmd(DYN_SLAVE, USART_DMAReq_Rx, ENABLE);
  dyn_slave_receiving_header=0x01;
}
